<?php


/**
 * @return mixed
 */
function ict_file_organizer_drush_command() {
  $items['file-organizer'] = [
    'description' => 'Move all known files from the root of the file system to directories organized by Year/month according to the file_managed timestamp. Be sure to run a backup of your site first!',
    'options' => [
      'dir' => 'Enter the sub-directory you want to target. Assumes the root of the public directory',
      //'fields' => 'Enter a comma separated list of field names you want to check. Assumes body. use the full name of the field as you see in the content type ui such as field_sidebar or field_highlight, etc.',
      'feedback' => 'Show the files being moved. Assumes false',
    ],
  ];
  return $items;
}


/**
 * @throws \Exception
 */
function drush_ict_file_organizer_file_organizer() {
  $dir = drush_get_option('dir', FALSE);
  //$fields = drush_get_option('fields', 'body');
  $field_table = 'field_data_body';
  $field_value = $field . '_value';
  $feedback = drush_get_option('feedback', FALSE);



  $query = db_select('file_managed', 'fm');
  if ($dir) {
    // in the target directory
    $query->condition('uri', 'public://' . $dir . '/%', 'LIKE');
    // but not in any directories in the target directory
    $query->condition('uri', 'public://' . $dir . '/%/%', 'NOT LIKE');
  }
  else {
    $query->condition('uri', 'public://%/%', 'NOT LIKE');
  }
  $query->fields('fm');
  $result = $query->execute();
  if ($feedback) {
    drush_print(dt('Will move @count files.', [
      '@count' => $result->rowCount(),
    ]));
  }
  if (drush_confirm('Make sure you run a backup before you complete this! Are you ready to continue?')) {
    drush_print('Here we go!');
  }
  else {
    return drush_user_abort();
  }

  foreach ($result as $item) {
    $source_file = $item->filename;
    if ($dir) {
      $source_uri = 'public://' . $dir . '/' . $source_file;
    }
    else {
      $source_uri = 'public://' . $source_file;
    }

    // new directory is based on the date of the file.
    $destination_dir = date('Y/m/', $item->timestamp);

    if ($dir) {
      $destination_dir = $dir . '/' . $destination_dir;
    }

    $destination_file = $destination_dir . $source_file;
    $uri = 'public://' . $destination_file;
    $directory_uri = 'public://' . $destination_dir;

    // prepare an update of the file record to point it to the $destination_dir
    // if the file move doesn't work the entire db update will fail. This is the most likely point of failure.
    // build a string of exceptions to report at the end of the process
    $exceptions = [];
    try {
      //check if the directory is there, if not create it.
      if (!file_prepare_directory($directory_uri, FILE_CREATE_DIRECTORY)) {
        drupal_mkdir($directory_uri, NULL, TRUE);
      }

      file_unmanaged_move($source_uri, $uri, FILE_EXISTS_REPLACE);

      $update_query = db_update('file_managed')
        ->condition('fid', $item->fid)
        ->fields([
          'uri' => $uri,
        ]);
      $update_query->execute();
      //search for all body fields for the $source_file and collect a list of entity_ids.
      $body_search = db_select($field_table, 'fb');
      $body_search->condition('fb.' . $field_value, '%' . $source_file . '%', 'LIKE');
      $body_search->fields('fb');
      $records = $body_search->execute();
      foreach ($records as $record) {
        // update node.
        $node = node_load($record->entity_id);
        $body_value = $node->$field[LANGUAGE_NONE][0]['value'];
        $new_value = str_replace($source_file, $destination_file, $body_value);
        $node->$field[LANGUAGE_NONE][0]['value'] = $new_value;
        node_save($node);
      }
      if ($feedback) {
        drush_print(dt('Moved @source to @destination.', [
          '@source' => $source_uri,
          '@destination' => $uri,
        ]));
      }
    } catch (\Exception $e) {
      $exceptions[] = $e->getMessage();
    }
  }
  if(!empty($exceptions)) {
    drush_print('The following errors occurred:');
    foreach ($exceptions as $exception) {
      drush_print($exception);
    }
  }
  else{
     drush_print('No errors occurred during the file organization process');
  }

}
